import { defineStore } from 'pinia'
import type { CourseDetail, CourseList, CourseWithThumbnail } from '#/Course'
import _axios from '@/plugins/axios'
import type { StrapiError, StrapiObject } from '#/index'
import type { AxiosError } from 'axios'
import { customMarked } from '@/plugins/marked'

export interface CourseStateStore {
  courseDisplayed: StrapiObject<CourseWithThumbnail> | null;
  courses: StrapiObject<CourseWithThumbnail>[];
}

export const useCourseStore = defineStore('Course', {
  state: (): CourseStateStore => ({
    courseDisplayed: null,
    courses: [],
  }),

  actions: {
    async loadCourse(id: number): Promise<void> {
      try {
        const response = await _axios.get<CourseDetail>(`/courses/${id}?populate[]=thumbnail`)
        this.courseDisplayed = {
          ...response.data.data,
          attributes: {
            ...response.data.data.attributes,
            abstract: customMarked(response.data.data.attributes.abstract),
            content: customMarked(response.data.data.attributes.content),
            thumbnail: {
              ...response.data.data.attributes.thumbnail,
              data: {
                ...response.data.data.attributes.thumbnail.data,
                attributes: {
                  ...response.data.data.attributes.thumbnail.data.attributes,
                  url: `${import.meta.env.VITE_APP_BACK_URL}${response.data.data.attributes.thumbnail.data.attributes.url}`,
                },
              },
            },
          },
        }
      } catch (error) {
        throw Error((error as AxiosError<StrapiError>).response?.data.error.message)
      }
    },

    async loadCourses(): Promise<void> {
      try {
        const response = await _axios.get<CourseList>('/courses?populate[]=thumbnail')
        this.courses = response.data.data.map(course => ({
          ...course,
          attributes: {
            ...course.attributes,
            abstract: customMarked(course.attributes.abstract),
            content: customMarked(course.attributes.content),
            thumbnail: {
              ...course.attributes.thumbnail,
              data: {
                ...course.attributes.thumbnail.data,
                attributes: {
                  ...course.attributes.thumbnail.data.attributes,
                  url: `${import.meta.env.VITE_APP_BACK_URL}${course.attributes.thumbnail.data.attributes.url}`,
                },
              },
            },
          },
        }))
      } catch (error) {
        throw Error((error as AxiosError<StrapiError>).response?.data.error.message)
      }
    },
  },
})
